//
//  main.cpp
//  mux_4_to_1
//
//  Created by Behrad Niazmand on 01/02/16.
//  Copyright © 2016 Behrad Niazmand. All rights reserved.
//

#include "mux_4_to_1.h"

SC_MODULE (driver)
{
    sc_out <bool> d_in0, d_in1, d_in2, d_in3;
    sc_out <sc_uint<2> > d_select;

    void prc_driver()
    {
        sc_uint<6> pattern;
        pattern = 0;
        
        while (1)
        {
            d_in0 = pattern[5];
            d_in1 = pattern[4];
            d_in2 = pattern[3];
            d_in3 = pattern[2];
            d_select = pattern.range(1,0);
            
            wait(10, SC_NS);
            
            pattern++;
        }
        
//        sc_stop();
    }
    
    SC_CTOR(driver) {
        SC_THREAD(prc_driver);
    }
};

SC_MODULE (monitor)
{
    sc_in <bool> m_in0, m_in1, m_in2, m_in3;
    sc_in <sc_uint<2> > m_select;
    sc_in <bool> m_out;
    
    void prc_monitor()
    {
        cout << "At time: " << sc_time_stamp() << ": " << "in0, in1, in2, in3, select = " << m_in0 << " " << m_in1 << " " << m_in2 << " " << m_in3 << " " << (sc_bv<2>) m_select << "\t";
        cout << "out = " << m_out << endl;
    }
    
    SC_CTOR(monitor) {
        SC_METHOD(prc_monitor);
        sensitive << m_in0 << m_in1 << m_in2 << m_in3 << m_select;
    }
};

int sc_main (int argc, char * argv[])
{
    sc_signal <bool> t_in0, t_in1, t_in2, t_in3;
    sc_signal <sc_uint<2> > t_select;
    sc_signal <bool> t_out;

    // DRIVER
    driver DRIVER ("DRIVER");
    DRIVER.d_in0    (t_in0);
    DRIVER.d_in1    (t_in1);
    DRIVER.d_in2    (t_in2);
    DRIVER.d_in3    (t_in3);
    DRIVER.d_select (t_select);
    
    // MONITOR
    monitor MONITOR ("MONITOR");
    MONITOR.m_in0 (t_in0);
    MONITOR.m_in1 (t_in1);
    MONITOR.m_in2 (t_in2);
    MONITOR.m_in3 (t_in3);
    MONITOR.m_select (t_select);
    MONITOR.m_out (t_out);

    // MAIN MODULE
    mux_4_to_1 MUX ("MUX_4_to_1");
    MUX.in0  (t_in0);
    MUX.in1  (t_in1);
    MUX.in2  (t_in2);
    MUX.in3  (t_in3);
    MUX.select  (t_select);
    MUX.out (t_out);
    
    // Dumping signals into .vcd file
    sc_trace_file *trace = sc_create_vcd_trace_file ("MUX_4_to_1");
    
    sc_trace (trace, t_in0                              , "in0");
    sc_trace (trace, t_in1                              , "in1");
    sc_trace (trace, t_in2                              , "in2");
    sc_trace (trace, t_in3                              , "in3");
    sc_trace (trace, t_select                           , "select");
    sc_trace (trace, t_out                              , "out");
			
    sc_start(640, SC_NS);
    
    sc_close_vcd_trace_file(trace);

    return 0;
}

